package bprecchiffreapp;

/**
 * Package bprecchiffreapp
 * Rseau BP et reconnaissance de chiffres.
 * Rseaux Neauronaux, Vuibert 2006.
 * Jean-Philippe Rennard
 * version 1.0, 17/3/2006
 */

import java.util.*;
import java.awt.*;
import baseui.*;

/**
 * <p>Title: Processus BP chiffres</p>
 * <p>Description: Gestion du processus.</p>
 */
public class BPRecChiffreProcess extends Cprocess {
  /** Taille de l'historique */
  public static int TAILLE_HISTO = 1000;

  /** Pointeur sur l'interface utilisateur */
  BPRecChiffreUI appUI;
  /** Vue */
  BPRecChiffreVue bpVue;
  /** Reseau de calcul */
  ReseauBPRecChiffre leReseau;
  /** Valeur d'erreur courante */
  double errCourante = 0.0D;
  /** Historique erreurs */
  public Vector histError;
  /** Erreur initiale */
  double errorInit;
  /** Affichage courant */
  double[] affCrt;

  /**
   * Constructeur du processus dans une interface utilisateur.
   * @param ui BPRecChiffreUI : interface utilisateur
   */
  public BPRecChiffreProcess(BPRecChiffreUI ui) {
    super(ui);
    appUI = ui;
    bpVue = new BPRecChiffreVue(this, Color.white, Color.black);
    vue = (Cvue)bpVue;
    tempoProcess = 1;
  }

  /** Initialisation du rseau */
  public void initReseau() {
    generation = 0;
    histError = new Vector(appUI.tailleHisto);
    errCourante = 0;
    affCrt = new double[7];
    leReseau = new ReseauBPRecChiffre(this, appUI.nbCouchesCachees,
                                      appUI.tailleCouchesCachees,
                                      appUI.eta, appUI.moment);
  }

  /** Rinitialisation du rseau */
  public void reInit() {
    initReseau();
    initVue();
  }

  /** Initialisation/MAJ de la vue */
  public void initVue() {
    bpVue.setNewSize();
  }

  /** Apprentissage */
  public void geneSuiv() {
    errCourante = leReseau.iteration();
    if (generation == 1) errorInit = errCourante;
    if(generation > appUI.tailleHisto) {
      histError.removeElementAt(0);
    }
    histError.addElement(new Double(errCourante));
    if(generation > leReseau.maxIteration
       || errCourante < leReseau.seuilConvergence) {
      appUI.stopGo();
      return;
    }
    bpVue.afficheGraph();
    bpVue.repaint();
    appUI.affGeneration();
    generation++;
  }

  /** Identification */
  public int identifie() {
    int val = leReseau.reconnaissance(affCrt);
    bpVue.afficheResultat(val);
    return val;
  }
}
