package bprecchiffreapp;

/**
 * Package bprecchiffreapp
 * Rseau BP et reconnaissance de chiffres.
 * Rseaux Neauronaux, Vuibert 2006.
 * Jean-Philippe Rennard
 * version 1.0, 17/3/2006
 */

import java.awt.*;
import java.awt.event.*;
import baseui.*;

/**
 * <p>Title: Interface utilisateur</p>
 * <p>Description: Interface utilisateur reconnaissance se chiffres.</p>
 */
public class BPRecChiffreUI extends Cui { // Frame Cui

  /** Applet */
  public BPRecChiffre applet;
  /** processus */
  public BPRecChiffreProcess processBP;
  /** Nb couches caches */
  int nbCouchesCachees;
  /** Nb neurones couches caches */
  int[] tailleCouchesCachees;
  /** Paramtre Eta */
  double eta;
  /** Moment */
  double moment;
  /** Taille de l'historique du graphe d'apprentissage */
  int tailleHisto = BPRecChiffreProcess.TAILLE_HISTO;

  BorderLayout borderLayout1 = new BorderLayout();
  Label statusBar = new Label();
  Panel panel1 = new Panel();
  BorderLayout borderLayout2 = new BorderLayout();
  Panel panel2 = new Panel();
  Button btnPas = new Button();
  GridLayout gridLayout1 = new GridLayout();
  Button btnGo = new Button();
  Button btnAbout = new Button();
  Panel panelVue = new Panel();
  Panel panel3 = new Panel();
  Label label1 = new Label();
  GridLayout gridLayout2 = new GridLayout();
  TextField txtCouches = new TextField();
  Label label2 = new Label();
  TextField txtEta = new TextField();
  Label label3 = new Label();
  TextField txtMoment = new TextField();
  Label label4 = new Label();
  TextField txtHisto = new TextField();
  GridLayout gridLayout3 = new GridLayout();
  Button btnRecon = new Button();
  Button btnModif = new Button();
  Button btnReset = new Button();


  public BPRecChiffreUI(BPRecChiffre ap) {
    applet = ap;
    processBP = new BPRecChiffreProcess(this);
    process = (Cprocess) processBP;
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    this.setTitle(applet.comments + " " + applet.auteur);
    this.addMouseListener(new BPRecChiffreUI_this_mouseAdapter(this));
    this.addWindowListener(new BPRecChiffreUI_this_windowAdapter(this));
    statusBar.setLocale(java.util.Locale.getDefault());
    statusBar.setText("Itration :");
    this.setLayout(borderLayout1);
    panel1.setLayout(borderLayout2);
    btnPas.setLabel("Pas  pas");
    btnPas.addActionListener(new BPRecChiffreUI_btnPas_actionAdapter(this));
    panel2.setLayout(gridLayout1);
    btnGo.setLabel("Apprend");
    btnGo.addActionListener(new BPRecChiffreUI_btnGo_actionAdapter(this));
    btnAbout.setLabel("A Propos");
    btnAbout.addActionListener(new BPRecChiffreUI_btnAbout_actionAdapter(this));
    label1.setText("Neur. cachs");
    panel3.setLayout(gridLayout2);
    label2.setText("Eta");
    label3.setText("Moment");
    label4.setText("Historique");
    panelVue.setLayout(gridLayout3);
    panelVue.addComponentListener(new BPRecChiffreUI_panelVue_componentAdapter(this));
    btnRecon.setLabel("Identifie");
    btnModif.setLabel("Modif.");
    btnModif.addActionListener(new BPRecChiffreUI_btnModif_actionAdapter(this));
    txtEta.setEnabled(false);
    txtEta.setText(Double.toString(ReseauBPRecChiffre.ETA));
    txtCouches.setEnabled(false);
    txtCouches.setText(Integer.toString(ReseauBPRecChiffre.TAILLE_COUCHES_CACHEES[0]));
    txtMoment.setEnabled(false);
    txtMoment.setText(Double.toString(ReseauBPRecChiffre.MOMENT));
    txtHisto.setEnabled(false);
    btnReset.setLabel("Reset");
    btnReset.addActionListener(new BPRecChiffreUI_btnReset_actionAdapter(this));
    this.add(statusBar, BorderLayout.SOUTH);
    this.add(panel1, BorderLayout.CENTER);
    panel1.add(panel2, BorderLayout.SOUTH);
    panel2.add(btnReset, null);
    panel2.add(btnGo, null);
    panel2.add(btnPas, null);
    panel1.add(panelVue, BorderLayout.CENTER);
    this.add(panel3, BorderLayout.NORTH);
    panel3.add(label1, null);
    panel3.add(txtCouches, null);
    panel3.add(label2, null);
    panel3.add(txtEta, null);
    panel3.add(label3, null);
    panel3.add(txtMoment, null);
    panel3.add(label4, null);
    panel3.add(txtHisto, null);
    txtHisto.setText(Integer.toString(tailleHisto));
    panel3.add(btnModif, null);
    // ajout de la vue
    panelVue.add(processBP.bpVue, null);
    panel2.add(btnRecon, null);
    panel2.add(btnAbout, null);
    initParam();
    processBP.initReseau();
    btnRecon.addActionListener(new BPRecChiffreUI_btnRecon_actionAdapter(this));

  }

  /** (In)active l'interface */
  public void setInterface(boolean e) {
    btnModif.setEnabled(e);
    btnPas.setEnabled(e);
    btnGo.setEnabled(e);
    btnAbout.setEnabled(e);
    btnRecon.setEnabled(e);
    btnReset.setEnabled(e);
  }

  /** Affichage de la gnration */
  public void affGeneration() {
    statusBar.setText(" Itration :"
                      + Integer.toString(processBP.generation)
                      + " Erreur : " + Double.toString(processBP.errCourante));

  }

  /** Display Info box */
  void dispInfoBox() {
    String s =  applet.product + " "
        + applet.version + "\n\n"
        + applet.comments + "\n"
        + applet.auteur + "\n"
        + applet.editeur +"\n"
        + applet.copyright + "\n\n"
        + applet.GNU1 + "\n" + applet.GNU2 + "\n" + applet.GNU3 + "\n";

    CaboutBox aboutBox = new CaboutBox(this, "Reconnaissance de chiffres", true);
    aboutBox.dispInfoBox(s);
  }

  void btnAbout_actionPerformed(ActionEvent e) {
    dispInfoBox();
  }

  void panelVue_componentResized(ComponentEvent e) {
    redimUi();
  }

  void this_windowClosing(WindowEvent e) {
    dispose();
  }

  void btnGo_actionPerformed(ActionEvent e) {
    stopGo();
  }

  /** Stop/Go */
  public void stopGo() {
    if (processBP.isRunning == false) {
      btnGo.setLabel("Stop");
      setInterface(false);
      btnGo.setEnabled(true);
      processBP.lancement();
    }
    else {
      btnGo.setLabel("Apprend");
      processBP.stop();
      setInterface(true);
    }
  }

  /** Pas  pas */
  void btnPas_actionPerformed(ActionEvent e) {
    processBP.geneSuiv();
  }

  /** Saisie et reconnaissance */
  void btnRecon_actionPerformed(ActionEvent e) {
    processBP.identifie();
  }

  /** Initialisation/MAJ paramtres rseau */
  public void initParam() {
    Calert x;
    String msg = "Certains paramtres n'taient pas corrects. Ils ont t remplacs par la valeur par dfaut.";
    int nbc = 1;
    int[] tc;
    boolean valide = true;
    boolean ttValide = true;
    String tcc;
    int n = 0;
    double d = 0.0D;
    // Taille couches caches. Les tailles de couches sont spares par une virgule
    tcc = txtCouches.getText();
    String s;
    int r = 0;
    int c = 0;
    for (int i=0; i<tcc.length(); i++) {
      if(tcc.charAt(i) == ',') nbc++;
    }
    tc = new int[nbc];
    do {
      s = "";
      do {
        s += tcc.charAt(r++);
      } while(r < tcc.length() && tcc.charAt(r) != ',' );
      try {
        tc[c] = Integer.parseInt(s);
      } catch(NumberFormatException er) {
        valide = false;
      }
      if(tc[c] < 1 || tc[c]>ReseauBPRecChiffre.MAX_NEURONES) valide = false;
      c++;
      r++;
    } while(c < nbc);
    if(!valide) {
      txtCouches.setText(Integer.toString(ReseauBPRecChiffre.TAILLE_COUCHES_CACHEES[0]));
      nbc = ReseauBPRecChiffre.NB_COUCHES_CACHEES;
      tc = ReseauBPRecChiffre.TAILLE_COUCHES_CACHEES;
      ttValide = false;
    }
    nbCouchesCachees = nbc;
    tailleCouchesCachees = tc;
    // Eta
    valide = true;
    try {
      d = Double.valueOf(txtEta.getText()).doubleValue();
    } catch(NumberFormatException er) {
      valide=false;
    }
    if(d < 0 || d > 1) valide = false;
    if(!valide) {
      txtEta.setText(Double.toString(ReseauBPRecChiffre.ETA));
      d = ReseauBPRecChiffre.ETA;
      ttValide = false;
    }
    eta = d;
    // Moment
    valide = true;
    try {
      d = Double.valueOf(txtMoment.getText()).doubleValue();
    } catch(NumberFormatException er) {
      valide=false;
    }
    if(d < 0 || d > 1) valide = false;
    if(!valide) {
      txtMoment.setText(Double.toString(ReseauBPRecChiffre.MOMENT));
      d = ReseauBPRecChiffre.MOMENT;
      ttValide = false;
    }
    moment = d;
    if(!ttValide) {
      x = new Calert(this, msg);
    }
    // Taille historique graphe
    valide = true;
    try {
      n = Integer.parseInt(txtHisto.getText());
    } catch(NumberFormatException er) {
      valide=false;
    }
    if(d <= 0 || d > 5000) valide = false;
    if(!valide) {
      txtHisto.setText(Integer.toString(BPRecChiffreProcess.TAILLE_HISTO));
      n = BPRecChiffreProcess.TAILLE_HISTO;
      ttValide = false;
    }
    tailleHisto = n;
    if(!ttValide) {
      x = new Calert(this, msg);
    }
  }

  /** Modification des paramtres rseau */
  void btnModif_actionPerformed(ActionEvent e) {
    if(btnModif.getLabel() == "Modif.") {
      btnModif.setLabel("Valide");
      txtCouches.setEnabled(true);
      txtEta.setEnabled(true);
      txtMoment.setEnabled(true);
      txtHisto.setEnabled(true);
      setInterface(false);
    }
    else {
      btnModif.setLabel("Modif.");
      txtCouches.setEnabled(false);
      txtEta.setEnabled(false);
      txtMoment.setEnabled(false);
      txtHisto.setEnabled(false);
      setInterface(true);
      initParam();
      processBP.reInit();
      affGeneration();
    }
    btnModif.setEnabled(true);
  }

  /** Rinitialisation */
  void btnReset_actionPerformed(ActionEvent e) {
    if (processBP.isRunning == true) {
      btnGo.setLabel("Apprend");
      processBP.stop();
      setInterface(true);
      processBP.reInit();
      processBP.lancement();
      btnGo.setLabel("Stop");
    }
    else {
      initParam();
      processBP.reInit();
      affGeneration();
    }
  }
}

class BPRecChiffreUI_btnAbout_actionAdapter implements java.awt.event.ActionListener {
  BPRecChiffreUI adaptee;

  BPRecChiffreUI_btnAbout_actionAdapter(BPRecChiffreUI adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.btnAbout_actionPerformed(e);
  }
}

class BPRecChiffreUI_panelVue_componentAdapter extends java.awt.event.ComponentAdapter {
  BPRecChiffreUI adaptee;

  BPRecChiffreUI_panelVue_componentAdapter(BPRecChiffreUI adaptee) {
    this.adaptee = adaptee;
  }
  public void componentResized(ComponentEvent e) {
    adaptee.panelVue_componentResized(e);
  }
}

class BPRecChiffreUI_this_windowAdapter extends java.awt.event.WindowAdapter {
  BPRecChiffreUI adaptee;

  BPRecChiffreUI_this_windowAdapter(BPRecChiffreUI adaptee) {
    this.adaptee = adaptee;
  }
  public void windowClosing(WindowEvent e) {
    adaptee.this_windowClosing(e);
  }
}

class BPRecChiffreUI_btnGo_actionAdapter implements java.awt.event.ActionListener {
  BPRecChiffreUI adaptee;

  BPRecChiffreUI_btnGo_actionAdapter(BPRecChiffreUI adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.btnGo_actionPerformed(e);
  }
}

class BPRecChiffreUI_btnPas_actionAdapter implements java.awt.event.ActionListener {
  BPRecChiffreUI adaptee;

  BPRecChiffreUI_btnPas_actionAdapter(BPRecChiffreUI adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.btnPas_actionPerformed(e);
  }
}

class BPRecChiffreUI_btnRecon_actionAdapter implements java.awt.event.ActionListener {
  BPRecChiffreUI adaptee;

  BPRecChiffreUI_btnRecon_actionAdapter(BPRecChiffreUI adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.btnRecon_actionPerformed(e);
  }
}

class BPRecChiffreUI_this_mouseAdapter extends java.awt.event.MouseAdapter {
  BPRecChiffreUI adaptee;

  BPRecChiffreUI_this_mouseAdapter(BPRecChiffreUI adaptee) {
    this.adaptee = adaptee;
  }
}

class BPRecChiffreUI_btnModif_actionAdapter implements java.awt.event.ActionListener {
  BPRecChiffreUI adaptee;

  BPRecChiffreUI_btnModif_actionAdapter(BPRecChiffreUI adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.btnModif_actionPerformed(e);
  }
}

class BPRecChiffreUI_btnReset_actionAdapter implements java.awt.event.ActionListener {
  BPRecChiffreUI adaptee;

  BPRecChiffreUI_btnReset_actionAdapter(BPRecChiffreUI adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.btnReset_actionPerformed(e);
  }
}
