package bprecchiffreapp;

/**
 * Package bprecchiffreapp
 * Rseau BP et reconnaissance de chiffres.
 * Rseaux Neauronaux, Vuibert 2006.
 * Jean-Philippe Rennard
 * version 1.0, 17/3/2006
 */

import java.awt.*;

import baseui.*;

/**
 * <p>Title: Vue de l'applet BP </p>
 * <p>Description: Interface graphique de l'applet reconnaissance de chiffres.</p>
 */
public class BPRecChiffreVue extends Cvue {
  /** Offset repre courbe */
  public int OffsetVertical = (int)(0.03 * lHauteur);
  public int OffsetHorizontal = (int)(0.03 * lLargeur);
  /** Pointeur sur processus */
  BPRecChiffreProcess processBP;
  /** Position X affichage rsultat */
  int posXAffRes = 0;
  /** Position Y affichage rsultat */
  int posYAffRes = 0;

  /** Constructeur par dfaut */
  public BPRecChiffreVue(BPRecChiffreProcess p) {
    super(Color.black, Color.white, Cvue.REPRANISOTROPIC);
    processBP = p;
  }

  /** Constructeur avec couleurs */
  public BPRecChiffreVue(BPRecChiffreProcess p, Color c, Color f) {
    super(c, f, Cvue.REPRANISOTROPIC);
    processBP = p;
  }

  /** Affichage du fond */
  public void afficheFond() {
    effaceFond();
    grBuff.drawLine(lToRX(0), lToRY((int) (0.5*lHauteur)-OffsetVertical),
                    lToRX(lLargeur), lToRY((int) (0.5*lHauteur)-OffsetVertical));
    afficheFondApprend();
    afficheFondRecon();
  }

  /** Fond pour l'apprentissage */
  public void afficheFondApprend() {
    grBuff.drawLine(lToRX(OffsetHorizontal), lToRY(lHauteur-OffsetVertical),
                    lToRX(lLargeur-OffsetHorizontal), lToRY(lHauteur-OffsetVertical));
    grBuff.drawLine(lToRX(OffsetHorizontal), lToRY(lHauteur-OffsetVertical),
                    lToRX(OffsetHorizontal), lToRY((int) (0.5 * (lHauteur-OffsetVertical))));
  }

  /** Fond pour la reconnaissance */
  public void afficheFondRecon() {
    float baseX;
    float baseY;
    Color colCrt = grBuff.getColor();
    int[] pos = new int[4];
    for(int i=1; i<8; i++) {
      pos = getPosSegment(i, pos);
      grBuff.setColor(processBP.affCrt[i-1] == 1.0D ? Color.red : Color.white);
      grBuff.fillRect(pos[0], pos[1], pos[2], pos[3]);
    }
    grBuff.setColor(colCrt);
    int sizeAct = grBuff.getFont().getSize();
    grBuff.setFont(new Font(grBuff.getFont().getName(), Font.PLAIN, 20));
   for(int i=0; i<4;i++) {
     for(int j=0; j<4;j++) {
       baseX = OffsetHorizontal + (lLargeur / 12.0F) * j;
       baseY = OffsetVertical + rToLY((int)((grBuff.getFontMetrics().getHeight()) * 0.5)) +  (lHauteur / 8.0F) * i;
       if (4 * i + j < 10) {
         grBuff.drawString(Integer.toString(4 * i + j) ,lToRX(baseX),lToRY(baseY));
       }
       if (4 * i + j == 10) {
         grBuff.drawString("A" ,lToRX(baseX),lToRY(baseY));
       }
       if (4 * i + j == 11) {
         grBuff.drawString("b" ,lToRX(baseX),lToRY(baseY));
       }
       if (4 * i + j == 12) {
         grBuff.drawString("C" ,lToRX(baseX),lToRY(baseY));
       }
       if (4 * i + j == 13) {
         grBuff.drawString("d" ,lToRX(baseX),lToRY(baseY));
       }
       if (4 * i + j == 14) {
         grBuff.drawString("E" ,lToRX(baseX),lToRY(baseY));
       }
       if (4 * i + j == 15) {
         grBuff.drawString("F" ,lToRX(baseX),lToRY(baseY));
       }
     }
   }
   grBuff.setFont(new Font(grBuff.getFont().getName(), Font.PLAIN, sizeAct));
  }

 /**
  * Calcul coordonnes segments selon rang
  * @param rang int : rang du segment (1-7)
  * @param pos int[] : Tableau des coordonnes (x, y, longueur, largeur)
  * @return int[] : Tableau des coordonnes
  */
 public int[] getPosSegment(int rang, int[] pos) {
    float longueurSegment = lHauteur * 0.025F;
    float epaisseurSegment = lLargeur * 0.01F;
    float baseX;
    float baseY;

    longueurSegment = lHauteur * 0.1F;
    epaisseurSegment = lLargeur * 0.02F;
    baseX = (lLargeur / 2 ) - longueurSegment / 2.0F;
    baseY = (((0.5F * lHauteur) - OffsetVertical)
             - (2.0F * longueurSegment + 3.0F * epaisseurSegment)) / 2.0F;

    pos[0] = lToRX(baseX);
    pos[1] = lToRY(baseY);
    switch(rang) {
      case 1:
        pos[2] = lToRX(longueurSegment);
        pos[3] = lToRY(epaisseurSegment);
        return pos;
      case 2:
        pos[0] += lToRX(longueurSegment) ;
        pos[1] += lToRY(epaisseurSegment);
        pos[2] = lToRX(epaisseurSegment);
        pos[3] = lToRY(longueurSegment);
        return pos;
      case 3:
        pos[0] += lToRX(longueurSegment) ;
        pos[1] += 2 * lToRY(epaisseurSegment) + lToRY(longueurSegment);
        pos[2] = lToRX(epaisseurSegment);
        pos[3] = lToRY(longueurSegment);
        return pos;
      case 4:
        pos[1] += 2 * lToRY(epaisseurSegment) + 2 * lToRY(longueurSegment);
        pos[2] = lToRX(longueurSegment);
        pos[3] = lToRY(epaisseurSegment);
        return pos;
      case 5:
        pos[0] -= lToRX(epaisseurSegment);
        pos[1] += 2 * lToRY(epaisseurSegment) + lToRY(longueurSegment);
        pos[2] = lToRX(epaisseurSegment);
        pos[3] = lToRY(longueurSegment);
        return pos;
      case 6:
        pos[0] -= lToRX(epaisseurSegment);
        pos[1] += lToRY(epaisseurSegment);
        pos[2] = lToRX(epaisseurSegment);
        pos[3] = lToRY(longueurSegment);
        return pos;
      case 7:
      default:
        pos[1] += lToRY(epaisseurSegment) + lToRY(longueurSegment);
        pos[2] = lToRX(longueurSegment);
        pos[3] = lToRY(epaisseurSegment);
        posXAffRes = pos[0] + pos[2] + lToRX(longueurSegment);
        posYAffRes = pos[1] + pos[3];
        return pos;
    }
  }

  /** Affiche rsultat reconnaissance */
  public void afficheResultat(int r) {
    processBP.appUI.redimUi();
    int sizeAct = grBuff.getFont().getSize();
    grBuff.setFont(new Font(grBuff.getFont().getName(), Font.PLAIN, 20));
    if (r == ReseauBPRecChiffre.NON_RECONNU) {
      grBuff.drawString("Non reconnu", posXAffRes, posYAffRes);
    }
    else {
      grBuff.drawString("C'est un : " + Integer.toString(r), posXAffRes, posYAffRes);
    }
    repaint();
    grBuff.setFont(new Font(grBuff.getFont().getName(), Font.PLAIN, sizeAct));
  }

  /** Affiche graphique d'erreur */
  public void afficheGraph() {
    double valCrt = 0;
    double valCrt2 = 0;
    afficheFond();
    if(processBP.generation == 0) return;
    double valMax = ((Double)(processBP.histError.elementAt(0))).doubleValue();
    int maxIt = processBP.generation < processBP.appUI.tailleHisto ?
           processBP.generation : processBP.appUI.tailleHisto;
    valMax = normErreur(processBP.errorInit);
    for(int i=1; i<maxIt; i++) {
      valCrt = (normErreur(((Double)(processBP.histError.elementAt(i))).doubleValue())
                *((0.5 * lHauteur) - 2*OffsetVertical))/valMax;
      if (i == 1) {
        valCrt2 = valCrt;
      }
      else {
        valCrt2 =
            (normErreur(((Double)(processBP.histError.elementAt(i-1))).doubleValue())
             * ((0.5 * lHauteur) - 2*OffsetVertical))/valMax;
      }
      grBuff.drawLine(lToRX((int)((i-1)*((lLargeur-2*OffsetHorizontal)/maxIt))
                            + OffsetHorizontal),
                      lToRY((int) (0.5 * lHauteur)
                            + ((int) (0.5 * lHauteur + 0*OffsetVertical)
                               - (int)(valCrt2+OffsetVertical))),
                      lToRX((int)(i*((lLargeur-2*OffsetHorizontal)/maxIt))
                            + OffsetHorizontal),
                      lToRY((int) (0.5 * lHauteur)
                            + ((int) (0.5 * lHauteur + 0*OffsetVertical)
                               - (int)(valCrt+OffsetVertical))));
    }
    grBuff.drawString(String.valueOf(
      ((Double)(processBP.histError.elementAt(0))).doubleValue()).substring(0, 4) ,
                      1, lToRY((int)(0.5 * lHauteur) + 1 * OffsetVertical));
    grBuff.drawString(String.valueOf(
      ((Double)(processBP.histError.elementAt(maxIt-1))).doubleValue()).substring(0,5) ,
                      1, lToRY((lHauteur - 2 * OffsetVertical)
                               - (int)valCrt+OffsetVertical));
    grBuff.drawString(String.valueOf(
      processBP.generation), lToRX(lLargeur-2*OffsetHorizontal), lToRY(lHauteur-1));
  }

  /** Slection d'un chiffre avec la souris */
  public boolean mouseDown(Event e,int x,int y) {
    float baseX;
    float baseY;
    double dist;
    int valCrt;

    Color colCrt = grBuff.getColor();
    int posx = (int)(rToLX(x));
    int posy = (int)(rToLY(y));
    valCrt = -1;
    for(int i=0; i<4;i++) {
      for (int j = 0; j < 4; j++) {
        baseX = OffsetHorizontal + (lLargeur / 12.0F) * j
            + grBuff.getFontMetrics().getHeight() / 2;
        baseY = OffsetVertical
            + rToLY((int)((grBuff.getFontMetrics().getHeight()) * 0.25)) +
            (lHauteur / 8.0F) * i;
        dist = Math.sqrt(Math.pow(posx - baseX, 2)) + Math.sqrt(Math.pow(posy - baseY, 2));
        if(dist < rToLY((int)((grBuff.getFontMetrics().getHeight())/ 2.0F))) {
          valCrt = 4 * i + j;
        }
        if(valCrt > 15 ) valCrt = -1;
      }
    }
    for(int i=0; i<7; i++) {
      processBP.affCrt[i] = 0.0D;
    }
    grBuff.setColor(Color.red);
    switch(valCrt) {
      case 0:
        processBP.affCrt[0] = 1.0D;
        processBP.affCrt[1] = 1.0D;
        processBP.affCrt[2] = 1.0D;
        processBP.affCrt[3] = 1.0D;
        processBP.affCrt[4] = 1.0D;
        processBP.affCrt[5] = 1.0D;
        break;
      case 1:
        processBP.affCrt[1] = 1.0D;
        processBP.affCrt[2] = 1.0D;
        break;
      case 2:
        processBP.affCrt[0] = 1.0D;
        processBP.affCrt[1] = 1.0D;
        processBP.affCrt[3] = 1.0D;
        processBP.affCrt[4] = 1.0D;
        processBP.affCrt[6] = 1.0D;
        break;
      case 3:
        processBP.affCrt[0] = 1.0D;
        processBP.affCrt[1] = 1.0D;
        processBP.affCrt[2] = 1.0D;
        processBP.affCrt[3] = 1.0D;
        processBP.affCrt[6] = 1.0D;
        break;
      case 4:
        processBP.affCrt[1] = 1.0D;
        processBP.affCrt[2] = 1.0D;
        processBP.affCrt[5] = 1.0D;
        processBP.affCrt[6] = 1.0D;
        break;
      case 5:
        processBP.affCrt[0] = 1.0D;
        processBP.affCrt[2] = 1.0D;
        processBP.affCrt[3] = 1.0D;
        processBP.affCrt[5] = 1.0D;
        processBP.affCrt[6] = 1.0D;
        break;
      case 6:
        processBP.affCrt[0] = 1.0D;
        processBP.affCrt[2] = 1.0D;
        processBP.affCrt[3] = 1.0D;
        processBP.affCrt[4] = 1.0D;
        processBP.affCrt[5] = 1.0D;
        processBP.affCrt[6] = 1.0D;
        break;
      case 7:
        processBP.affCrt[0] = 1.0D;
        processBP.affCrt[1] = 1.0D;
        processBP.affCrt[2] = 1.0D;
        break;
      case 8:
        processBP.affCrt[0] = 1.0D;
        processBP.affCrt[1] = 1.0D;
        processBP.affCrt[2] = 1.0D;
        processBP.affCrt[3] = 1.0D;
        processBP.affCrt[4] = 1.0D;
        processBP.affCrt[5] = 1.0D;
        processBP.affCrt[6] = 1.0D;
        break;
      case 9:
        processBP.affCrt[0] = 1.0D;
        processBP.affCrt[1] = 1.0D;
        processBP.affCrt[2] = 1.0D;
        processBP.affCrt[3] = 1.0D;
        processBP.affCrt[5] = 1.0D;
        processBP.affCrt[6] = 1.0D;
        break;
      case 10:
        processBP.affCrt[0] = 1.0D;
        processBP.affCrt[1] = 1.0D;
        processBP.affCrt[2] = 1.0D;
        processBP.affCrt[4] = 1.0D;
        processBP.affCrt[5] = 1.0D;
        processBP.affCrt[6] = 1.0D;
        break;
      case 11:
        processBP.affCrt[2] = 1.0D;
        processBP.affCrt[3] = 1.0D;
        processBP.affCrt[4] = 1.0D;
        processBP.affCrt[5] = 1.0D;
        processBP.affCrt[6] = 1.0D;
        break;
      case 12:
        processBP.affCrt[0] = 1.0D;
        processBP.affCrt[3] = 1.0D;
        processBP.affCrt[4] = 1.0D;
        processBP.affCrt[5] = 1.0D;
        break;
      case 13:
        processBP.affCrt[1] = 1.0D;
        processBP.affCrt[2] = 1.0D;
        processBP.affCrt[3] = 1.0D;
        processBP.affCrt[4] = 1.0D;
        processBP.affCrt[6] = 1.0D;
        break;
      case 14:
        processBP.affCrt[0] = 1.0D;
        processBP.affCrt[3] = 1.0D;
        processBP.affCrt[4] = 1.0D;
        processBP.affCrt[5] = 1.0D;
        processBP.affCrt[6] = 1.0D;
        break;
      case 15:
        processBP.affCrt[0] = 1.0D;
        processBP.affCrt[4] = 1.0D;
        processBP.affCrt[5] = 1.0D;
        processBP.affCrt[6] = 1.0D;
        break;
      default:
    }
    grBuff.setColor(colCrt);
    afficheGraph();
    repaint();
    return true;
  }

  /** Normalisation valeur d'erreur */
  private double normErreur(double e) {
    return Math.log(e*(1.0D/processBP.leReseau.seuilConvergence))/Math.log(10);
  }

  /** Changement de taille de la vue */
   public void setNewSize() {
     redimGrBuff();
     afficheGraph();
     repaint();
   }
}




